@@ -15,14 +15,17 @@ from transformers import (
     Trainer,
     TrainingArguments,
 )
-from transformers.data.processors.utils import InputFeatures, InputExample
-from torch.utils.data.dataset import Dataset
+from datasets import Dataset
 import torch
 import logging
 
 # intra-library imports
 from ..CnlpModelForClassification import CnlpModelForClassification, CnlpConfig
-from ..cnlp_data import cnlp_convert_examples_to_features
+from ..HierarchicalTransformer import HierarchicalModel
+from ..cnlp_data import cnlp_preprocess_data
+
+class UnannotatedDocument(BaseModel):
+    doc_text: str
 
 class EntityDocument(BaseModel):
     ''' doc_text: The raw text of the document
@@ -30,36 +33,29 @@ class EntityDocument(BaseModel):
     doc_text: str
     entities: List[List[int]]
 
-class ClassificationDocumentDataset(Dataset):
-    def __init__(self, features, label_list):
-        self.features = features
-        self.label_list = label_list
-
-    def __len__(self):
-        return len(self.features)
-
-    def __getitem__(self, i) -> InputFeatures:
-        return self.features[i]
-
-    def get_labels(self):
-        return self.label_list
-
-    @classmethod
-    def from_instance_list(cls, inst_list, tokenizer, label_list, max_length=128):
-        examples = []
-        for (ind,inst) in enumerate(inst_list):
-            guid = 'instance-%d' % (ind)
-            examples.append(InputExample(guid=guid, text_a=inst, text_b='', label=None))
-
-        features = cnlp_convert_examples_to_features(
-            examples,
-            tokenizer,
-            max_length=max_length,
-            label_list = label_list,
-            output_mode='classification',
-            inference=True
-        )
-        return cls(features, label_list)
+def get_dataset(inst_list, tokenizer, label_lists, tasks, max_length: int = 128, hier: bool = False, chunk_len: int = 200, num_chunks: int = 40, insert_empty_chunk_at_beginning: bool = False,):
+    dataset = Dataset.from_dict({'text':inst_list})
+    task_dataset = dataset.map(
+                    cnlp_preprocess_data,
+                    batched=True,
+                    load_from_cache_file=False,
+                    desc="Running tokenizer on dataset, organizing labels, creating hierarchical segments if necessary",
+                    batch_size=100,
+                    fn_kwargs = {
+                        'tokenizer':tokenizer,
+                        'max_length':max_length,
+                        'label_lists':label_lists,
+                        'inference': True,
+                        'hierarchical':hier,
+                        # TODO: need to get this from the model if necessary
+                        'chunk_len':chunk_len,
+                        'num_chunks':num_chunks,
+                        'insert_empty_chunk_at_beginning':insert_empty_chunk_at_beginning,
+                        'truncate_examples': True,
+                        'tasks': tasks,
+                }
+    )
+    return task_dataset
 
 def create_instance_string(doc_text: str, offsets : List[int]):
     start = max(0, offsets[0]-100)
@@ -96,5 +92,37 @@ def initialize_cnlpt_model(app, model_name, cuda=True, batch_size=8):
         compute_metrics=None,
     )
 
+def initialize_hier_model(app, model_name, cuda=True, batch_size=1):
+    AutoConfig.register("cnlpt", CnlpConfig)
+    AutoModel.register(CnlpConfig, HierarchicalModel)
 
+    config: CnlpConfig = AutoConfig.from_pretrained(model_name)
 
+    encoder_dim = config.hidden_size
+
+    ## TODO (#122) - replace this with the transformer config when we fix that issue. just use defaults for now.
+    config.hier_head_config = dict(
+        n_layers=2,
+        d_model=encoder_dim,
+        d_inner=2048,
+        n_head=8,
+        d_k=8,
+        d_v=96,
+    )
+
+    app.state.tokenizer = AutoTokenizer.from_pretrained(model_name,
+                                                  config=config)
+    
+    model = AutoModel.from_pretrained(model_name, cache_dir=os.getenv('HF_CACHE'), config=config)
+    model.train(False)
+
+    if cuda and not torch.cuda.is_available():
+        logging.warning('CUDA is set to True (probably a default) but was not available; setting to False and proceeding. If you have a GPU you need to debug why pytorch cannot see it.')
+        cuda = False
+    
+    if cuda:
+        model = model.to('cuda')
+    else:
+        model = model.to('cpu')
+
+    app.state.model = model
