@@ -42,17 +42,19 @@ from transformers.tokenization_utils import PreTrainedTokenizer
 from transformers.data.metrics import acc_and_f1
 from transformers.data.processors.utils import DataProcessor, InputExample, InputFeatures
 from transformers import ALL_PRETRAINED_CONFIG_ARCHIVE_MAP
-from transformers.optimization import AdamW, get_scheduler
+from transformers.optimization import get_scheduler
+from torch.optim import AdamW
 from transformers.trainer_pt_utils import get_parameter_names
 from transformers.file_utils import CONFIG_NAME
 from huggingface_hub import hf_hub_url
 
-from .cnlp_processors import cnlp_processors, cnlp_output_modes, cnlp_compute_metrics, tagging, relex, classification
+from .cnlp_processors import tagging, relex, classification
 from .cnlp_data import ClinicalNlpDataset, DataTrainingArguments
+from .cnlp_metrics import cnlp_compute_metrics
 
 from .CnlpModelForClassification import CnlpModelForClassification, CnlpConfig
 from .BaselineModels import CnnSentenceClassifier, LstmSentenceClassifier
-from .HierarchicalTransformer import HierarchicalModel, HierarchicalTransformerConfig
+from .HierarchicalTransformer import HierarchicalModel
 
 import requests
 
@@ -80,8 +82,8 @@ class CnlpTrainingArguments(TrainingArguments):
     final_task_weight: Optional[float] = field(
         default=1.0, metadata={"help": "Amount to up/down-weight final task in task list (other tasks weighted 1.0)"}
     )
-    freeze: bool = field(
-        default=False, metadata={"help": "Freeze the encoder layers and only train the layer between the encoder and classification architecture. Probably works best with --token flag since [CLS] may not be well-trained for anything in particular."}
+    freeze: float = field(
+        default=-1.0, metadata={"help": "Freeze the encoder layers and only train the layer between the encoder and classification architecture. Probably works best with --token flag since [CLS] may not be well-trained for anything in particular. If not specified, no weight freezing will be done. If specified as a flag (no arguments), 100%% of weights will be frozen. If a float (0..1.0) is specified, each weight will be frozen with that probability.", 'nargs':'?', "const":1.0 }
     )
     arg_reg: Optional[float] = field(
         default=-1, metadata={"help": "Weight to use on argument regularization term (penalizes end-to-end system if a discovered relation has low probability of being any entity type). Value < 0 (default) turns off this penalty."}
@@ -218,9 +220,16 @@ class ModelArguments:
             )
         },
     )
+    hier_dropout: Optional[float] = field(
+        default=0.1,
+        metadata={
+            "help": "For the hierarchical model, the dropout probability for the "
+                    "document-level transformer layers"
+        }
+    )
 
 
-def is_pretrained_model(model_name):
+def is_hub_model(model_name):
     # check if it's a model on the huggingface model hub:
     url = hf_hub_url(model_name, CONFIG_NAME)
     r = requests.head(url)
@@ -229,7 +238,8 @@ def is_pretrained_model(model_name):
 
     return False
 
-def main(json_file=None, json_obj=None):
+
+def main(json_file: Optional[str] = None, json_obj: Optional[Dict[str, Any]] = None):
     """
     See all possible arguments in :class:`transformers.TrainingArguments`
     or by passing the --help flag to this script.
@@ -246,6 +256,9 @@ def main(json_file=None, json_obj=None):
     :return: the evaluation results (will be empty if ``--do_eval`` not passed)
     """
     parser = HfArgumentParser((ModelArguments, DataTrainingArguments, CnlpTrainingArguments))
+    model_args: ModelArguments
+    data_args: DataTrainingArguments
+    training_args: CnlpTrainingArguments
 
     if json_file is not None and json_obj is not None:
         raise ValueError('cannot specify json_file and json_obj')
@@ -271,9 +284,6 @@ def main(json_file=None, json_obj=None):
             f"Output directory ({training_args.output_dir}) already exists and is not empty. Use --overwrite_output_dir to overcome."
         )
 
-    assert len(data_args.task_name) == len(data_args.data_dir), 'Number of tasks and data directories should be the same!'
-
-    
     # Setup logging
     logging.basicConfig(
         format="%(asctime)s - %(levelname)s - %(name)s -   %(message)s",
@@ -294,31 +304,6 @@ def main(json_file=None, json_obj=None):
     # Set seed
     set_seed(training_args.seed)
 
-    try:
-        task_names = []
-        num_labels = []
-        output_mode = []
-        tagger = []
-        relations = []
-        for task_name in data_args.task_name:
-            processor = cnlp_processors[task_name]()
-            if processor.get_num_tasks() > 1:
-                for subtask_num in range(processor.get_num_tasks()):
-                    task_names.append(task_name + "-" + processor.get_classifiers()[subtask_num])
-                    num_labels.append(len(processor.get_labels()))
-                    output_mode.append(classification)
-                    tagger.append(False)
-                    relations.append(False)
-            else:
-                task_names.append(task_name)
-                num_labels.append(len(processor.get_labels()))
-
-                output_mode.append(cnlp_output_modes[task_name])
-                tagger.append(cnlp_output_modes[task_name] == tagging)
-                relations.append(cnlp_output_modes[task_name] == relex)
-
-    except KeyError:
-        raise ValueError("Task not found: %s" % (data_args.task_name))
 
     # Load tokenizer: Need this first for loading the datasets
     tokenizer = AutoTokenizer.from_pretrained(
@@ -332,28 +317,42 @@ def main(json_file=None, json_obj=None):
     hierarchical = model_name == 'hier'
 
     # Get datasets
-    train_dataset = (
-        ClinicalNlpDataset(data_args, tokenizer=tokenizer, cache_dir=model_args.cache_dir, hierarchical=hierarchical) if training_args.do_train else None
-    )
-    eval_dataset = (
-        ClinicalNlpDataset(data_args, tokenizer=tokenizer, mode="dev", cache_dir=model_args.cache_dir, hierarchical=hierarchical)
-        if training_args.do_eval
-        else None
-    )
-    test_dataset = (
-        ClinicalNlpDataset(data_args, tokenizer=tokenizer, mode="test", cache_dir=model_args.cache_dir, hierarchical=hierarchical)
-        if training_args.do_predict
-        else None
+    dataset = (
+        ClinicalNlpDataset(data_args, tokenizer=tokenizer, cache_dir=model_args.cache_dir, hierarchical=hierarchical,)
     )
 
+    try:
+        task_names = []
+        num_labels = []
+        output_mode = []
+        tagger = []
+        relations = []
+        tasks_to_processors = {}
+        for dataset_ind in range(len(data_args.data_dir)):
+            processor = dataset.processors[dataset_ind]
+            # if processor.get_num_tasks() > 1:
+            for subtask_num in range(processor.get_num_tasks()):
+                task_names.append(processor.get_classifiers()[subtask_num])
+                num_labels.append(len(processor.get_labels()[task_names[-1]]))
+                task_output_mode = processor.get_output_mode(task_names[-1])
+                output_mode.append(task_output_mode)
+                tagger.append(task_output_mode == tagging)
+                relations.append(task_output_mode == relex)
+                tasks_to_processors[task_names[-1]] = processor
+
+                # tagger.append(False)
+                # relations.append(False)
+
+                tasks_to_processors[task_names[-1]] = processor
+
+    except KeyError:
+        raise ValueError("Task not found: %s" % (data_args.task_name))
+
     # Load pretrained model and tokenizer
     #
     # Distributed training:
     # The .from_pretrained methods guarantee that only one local process can concurrently
     # download model & vocab.
-
-    pretrained = False
-
     if model_name == 'cnn':
         model = CnnSentenceClassifier(len(tokenizer), 
                                       num_labels_list=num_labels,
@@ -376,55 +375,57 @@ def main(json_file=None, json_obj=None):
         if exists(model_path):
             model.load_state_dict(torch.load(model_path))
     elif model_name == 'hier':
-        # encoder_config = AutoConfig.from_pretrained(
-        #     model_args.config_name if model_args.config_name else model_args.encoder_name,
-        #     finetuning_task=data_args.task_name,
-        # )
-
-        pretrained = True
-
         encoder_name = model_args.config_name if model_args.config_name else model_args.encoder_name
-        config = CnlpConfig(
-            encoder_name,
-            data_args.task_name,
-            num_labels,
-            layer=model_args.layer,
-            tokens=model_args.token,
-            num_rel_attention_heads=model_args.num_rel_feats,
-            rel_attention_head_dims=model_args.head_features,
-            tagger=tagger,
-            relations=relations,
-        )
-        # num_tokens=len(tokenizer))
-        config.vocab_size = len(tokenizer)
+        if is_hub_model(encoder_name):
+            config = CnlpConfig(
+                encoder_name,
+                data_args.task_name,
+                num_labels,
+                layer=model_args.layer,
+                tokens=model_args.token,
+                num_rel_attention_heads=model_args.num_rel_feats,
+                rel_attention_head_dims=model_args.head_features,
+                tagger=tagger,
+                relations=relations,
+                hier_head_config=dict(
+                    n_layers=model_args.hier_num_layers,
+                    d_inner=model_args.hier_hidden_dim,
+                    n_head=model_args.hier_n_head,
+                    d_k=model_args.hier_d_k,
+                    d_v=model_args.hier_d_v,
+                    dropout=model_args.hier_dropout,
+                )
+            )
+            # num_tokens=len(tokenizer))
+            config.vocab_size = len(tokenizer)
 
-        encoder_dim = config.hidden_size
+            model = HierarchicalModel(
+                config=config,
+                class_weights=dataset.class_weights,
+                freeze=training_args.freeze,
+            )
+        else:
+            # use a checkpoint from an existing model
+            AutoConfig.register("cnlpt", CnlpConfig)
+            AutoModel.register(CnlpConfig, HierarchicalModel)
 
-        transformer_head_config = HierarchicalTransformerConfig(
-            n_layers=model_args.hier_num_layers,
-            d_model=encoder_dim,
-            d_inner=model_args.hier_hidden_dim,
-            n_head=model_args.hier_n_head,
-            d_k=model_args.hier_d_k,
-            d_v=model_args.hier_d_v,
-        )
+            config = AutoConfig.from_pretrained(
+                    encoder_name,
+                    cache_dir=model_args.cache_dir,
+                )
 
-        model = HierarchicalModel(
-            config=config,
-            transformer_head_config=transformer_head_config,
-            class_weights=None if train_dataset is None else train_dataset.class_weights,
-            final_task_weight=training_args.final_task_weight,
-            freeze=training_args.freeze,
-            argument_regularization=training_args.arg_reg,
-        )
+            ## TODO: check if user overwrote parameters in command line that could change behavior of the model and warn
+            #if data_args.chunk_len is not None:
 
+            logger.info("Loading pre-trained hierarchical model...")
+            model = AutoModel.from_pretrained(encoder_name, config=config)
     else:
         # by default cnlpt model, but need to check which encoder they want
         encoder_name = model_args.encoder_name
 
         # TODO check when download any pretrained language model to local disk, if 
-        # the following condition "is_pretrained_model(encoder_name)" works or not.
-        if not is_pretrained_model(encoder_name):
+        # the following condition "is_hub_model(encoder_name)" works or not.
+        if not is_hub_model(encoder_name):
             # we are loading one of our own trained models as a starting point.
             #
             # 1) if training_args.do_train is true:
@@ -463,7 +464,7 @@ def main(json_file=None, json_obj=None):
                         cache_dir=model_args.cache_dir,
                         tagger=tagger,
                         relations=relations,
-                        class_weights=None if train_dataset is None else train_dataset.class_weights,
+                        class_weights=dataset.class_weights,
                         final_task_weight=training_args.final_task_weight,
                         use_prior_tasks=model_args.use_prior_tasks,
                         argument_regularization=model_args.arg_reg)
@@ -478,7 +479,7 @@ def main(json_file=None, json_obj=None):
                 model = CnlpModelForClassification.from_pretrained(
                     model_args.encoder_name,
                     config=config,
-                    class_weights=None if train_dataset is None else train_dataset.class_weights,
+                    class_weights=dataset.class_weights,
                     final_task_weight=training_args.final_task_weight,
                     freeze=training_args.freeze,
                     bias_fit=training_args.bias_fit,
@@ -500,10 +501,9 @@ def main(json_file=None, json_obj=None):
                                 relations=relations,)
                                 #num_tokens=len(tokenizer))
             config.vocab_size = len(tokenizer)
-            pretrained = True
             model = CnlpModelForClassification(
                 config=config,
-                class_weights=None if train_dataset is None else train_dataset.class_weights,
+                class_weights=dataset.class_weights,
                 final_task_weight=training_args.final_task_weight,
                 freeze=training_args.freeze,
                 bias_fit=training_args.bias_fit,
@@ -518,7 +518,9 @@ def main(json_file=None, json_obj=None):
     )
 
     if training_args.do_train:
-        batches_per_epoch = math.ceil(len(train_dataset) / training_args.train_batch_size)
+        # TODO: This assumes that if there are multiple training sets, they all have the same length, but
+        # in the future it would be nice to be able to have multiple heterogeneous datasets
+        batches_per_epoch = math.ceil(dataset.num_train_instances / training_args.train_batch_size)
         total_steps = int(training_args.num_train_epochs * batches_per_epoch // training_args.gradient_accumulation_steps)
 
         if training_args.evals_per_epoch > 0:
@@ -533,16 +535,13 @@ def main(json_file=None, json_obj=None):
             logger.info('Evaluation strategy not specified so evaluating every epoch')
             training_args.evaluation_strategy = IntervalStrategy.EPOCH
 
-    def build_compute_metrics_fn(task_names: List[str], model) -> Callable[[EvalPrediction], Dict]:
+    def build_compute_metrics_fn(task_names: List[str], model, processors: Dict[str,DataProcessor]) -> Callable[[EvalPrediction], Dict]:
         def compute_metrics_fn(p: EvalPrediction):
 
             metrics = {}
             task_scores = []
             task_label_ind = 0
 
-            # if not p is list:
-            #     p = [p]
-
             for task_ind,task_name in enumerate(task_names):
                 if tagger[task_ind]:
                     preds = np.argmax(p.predictions[task_ind], axis=2)
@@ -552,21 +551,21 @@ def main(json_file=None, json_obj=None):
                 else:
                     preds = np.argmax(p.predictions[task_ind], axis=1)
 
-                if len(task_names) == 1:
-                    labels = p.label_ids[:,0]
-                elif relations[task_ind]:
-                    labels = p.label_ids[:,0,task_label_ind:task_label_ind+data_args.max_seq_length,:].squeeze()
+                if relations[task_ind]:
+                    # relation labels
+                    labels = p.label_ids[:,:,task_label_ind:task_label_ind+data_args.max_seq_length].squeeze()
                     task_label_ind += data_args.max_seq_length
-                elif p.label_ids.ndim == 4:
-                    labels = p.label_ids[:,0,task_label_ind:task_label_ind+1,:].squeeze()
-                    task_label_ind += 1
                 elif p.label_ids.ndim == 3:
-                    labels = p.label_ids[:,0,task_label_ind:task_label_ind+1].squeeze()
+                    labels = p.label_ids[:,:, task_label_ind:task_label_ind+1].squeeze()
                     task_label_ind += 1
+                elif p.label_ids.ndim == 2:
+                    labels = p.label_ids[:,task_ind].squeeze()
 
-                metrics[task_name] = cnlp_compute_metrics(task_name, preds, labels)
-                processor = cnlp_processors.get(task_name, cnlp_processors.get(task_name.split('-')[0], None))()
-                task_scores.append(processor.get_one_score(metrics.get(task_name, metrics.get(task_name.split('-')[0], None))))
+                processor = processors[task_name]
+                metrics[task_name] = cnlp_compute_metrics(task_name, task_ind, preds, labels, processor, processor.get_output_mode(task_name))
+                # FIXME - Defaulting to accuracy for model selection score, when it should be task-specific
+                task_scores.append( metrics[task_name].get('one_score', np.mean(metrics[task_name].get('f1'))))
+                #task_scores.append(processor.get_one_score(metrics.get(task_name, metrics.get(task_name.split('-')[0], None))))
 
             one_score = sum(task_scores) / len(task_scores)
 
@@ -580,9 +579,9 @@ def main(json_file=None, json_obj=None):
                             tokenizer.save_pretrained(training_args.output_dir)
                         for task_ind,task_name in enumerate(metrics):
                             with open(output_eval_file, "w") as writer:
-                                logger.info("***** Eval results for task %s *****" % (task_name))
+                                # logger.info("***** Eval results for task %s *****" % (task_name))
                                 for key, value in metrics[task_name].items():
-                                    logger.info("  %s = %s", key, value)
+                                    # logger.info("  %s = %s", key, value)
                                     writer.write("%s = %s\n" % (key, value))
                     model.best_score = one_score
                     model.best_eval_results = metrics
@@ -595,9 +594,9 @@ def main(json_file=None, json_obj=None):
     trainer = Trainer(
         model=model,
         args=training_args,
-        train_dataset=train_dataset,
-        eval_dataset=eval_dataset,
-        compute_metrics=build_compute_metrics_fn(task_names, model),
+        train_dataset=dataset.datasets[0].get('train', None),
+        eval_dataset=dataset.datasets[0].get('validation', None),
+        compute_metrics=build_compute_metrics_fn(task_names, model, tasks_to_processors),
     )
 
     # Training
@@ -617,7 +616,7 @@ def main(json_file=None, json_obj=None):
     eval_results = {}
     if training_args.do_eval:
         logger.info("*** Evaluate ***")
-
+        eval_dataset=dataset.datasets[0]['validation']
         try:
             eval_result = model.best_eval_results
         except:
@@ -633,54 +632,60 @@ def main(json_file=None, json_obj=None):
             with open(output_eval_predictions, 'w') as writer:
                 #Chen wrote the below but it doesn't work for all settings
                 predictions = trainer.predict(test_dataset=eval_dataset).predictions
-                dataset_labels = eval_dataset.get_labels()
-                for task_ind, task_name in enumerate(task_names):
-                    if output_mode[task_ind] == classification:
-                        task_predictions = np.argmax(predictions[task_ind], axis=1)
-                        for index, item in enumerate(task_predictions):
-                            if len(task_names) > len(dataset_labels):
-                                subtask_ind = 0
-                            else:
-                                subtask_ind = task_ind
-                            item = dataset_labels[subtask_ind][item]
-                            writer.write("Task %d (%s) - Index %d - %s\n" % (task_ind, task_name, index, item))
-                    elif output_mode[task_ind] == tagging:
-                        task_predictions = np.argmax(predictions[task_ind], axis=2)
-                        task_labels = dataset_labels[task_ind]
-                        for index, pred_seq in enumerate(task_predictions):
-                            wpind_to_ind = {}
-                            chunk_labels = []
+                dataset_labels = dataset.get_labels()
+                for dataset_ind in range(len(data_args.data_dir)):
+                    processor = dataset.processors[dataset_ind]
+                    for task_ind in range(processor.get_num_tasks()):
+                        task_name = processor.get_classifiers()[task_ind]
+                # for task_ind, task_name in enumerate(task_names):
+                        if output_mode[task_ind] == classification:
+                            task_predictions = np.argmax(predictions[task_ind], axis=1)
+                            for index, item in enumerate(task_predictions):
+                                if len(task_names) > len(dataset_labels):
+                                    subtask_ind = 0
+                                else:
+                                    subtask_ind = task_ind
+                                item = dataset_labels[dataset_ind][task_name][item]
+                                writer.write("Task %d (%s) - Index %d - %s\n" % (task_ind, task_name, index, item))
+                        elif output_mode[task_ind] == tagging:
+                            task_predictions = np.argmax(predictions[task_ind], axis=2)
+                            task_labels = dataset_labels[dataset_ind][task_name]
+                            for index, pred_seq in enumerate(task_predictions):
+                                wpind_to_ind = {}
+                                chunk_labels = []
 
-                            tokens = tokenizer.convert_ids_to_tokens(eval_dataset.features[index].input_ids)
-                            for token_ind in range(1,len(tokens)):
-                                if eval_dataset[index].input_ids[token_ind] <= 2:
-                                    break
-                                if tokens[token_ind].startswith('Ġ'):
-                                    wpind_to_ind[token_ind] = len(wpind_to_ind)
-                                    chunk_labels.append(task_labels[task_predictions[index][token_ind]])
+                                token_inds = eval_dataset['input_ids'][index]
+                                tokens = tokenizer.convert_ids_to_tokens(token_inds)
+                                for token_ind in range(1,len(tokens)):
+                                    if token_inds[token_ind] <= 2:
+                                        break
+                                    if tokens[token_ind].startswith('Ġ'):
+                                        wpind_to_ind[token_ind] = len(wpind_to_ind)
+                                        chunk_labels.append(task_labels[task_predictions[index][token_ind]])
 
-                            entities = get_entities(chunk_labels)
-                            writer.write('Task %d (%s) - Index %d: %s\n' % (task_ind, task_name, index, str(entities)))
-                    elif output_mode[task_ind] == relex:
-                        task_predictions = np.argmax(predictions[task_ind], axis=3)
-                        task_labels = dataset_labels[task_ind]
-                        assert task_labels[0] == 'None', 'The first labeled relation category should always be "None" but for task %s it is %s' % (task_names[task_ind], task_labels[0])
-                        
-                        for inst_ind in range(task_predictions.shape[0]):
-                            inst_preds = task_predictions[inst_ind]
-                            a1s, a2s = np.where(inst_preds > 0)
-                            for arg_ind in range(len(a1s)):
-                                a1_ind = a1s[arg_ind]
-                                a2_ind = a2s[arg_ind]
-                                cat = task_labels[ inst_preds[a1_ind][a2_ind] ]
-                                writer.write("Task %d (%s) - Index %d - %s(%d, %d)\n" % (task_ind, task_name, inst_ind, cat, a1_ind, a2_ind))
-                    else:
-                        raise NotImplementedError('Writing predictions is not implemented for this output_mode!')
+                                entities = get_entities(chunk_labels)
+                                writer.write('Task %d (%s) - Index %d: %s\n' % (task_ind, task_name, index, str(entities)))
+                        elif output_mode[task_ind] == relex:
+                            task_predictions = np.argmax(predictions[task_ind], axis=3)
+                            task_labels = dataset_labels[dataset_ind][task_name]
+                            # assert task_labels[0] == 'None', 'The first labeled relation category should always be "None" but for task %s it is %s' % (task_names[task_ind], task_labels[0])
+                            
+                            for inst_ind in range(task_predictions.shape[0]):
+                                inst_preds = task_predictions[inst_ind]
+                                a1s, a2s = np.where(inst_preds > 0)
+                                for arg_ind in range(len(a1s)):
+                                    a1_ind = a1s[arg_ind]
+                                    a2_ind = a2s[arg_ind]
+                                    cat = task_labels[ inst_preds[a1_ind][a2_ind] ]
+                                    writer.write("Task %d (%s) - Index %d - %s(%d, %d)\n" % (task_ind, task_name, inst_ind, cat, a1_ind, a2_ind))
+                        else:
+                            raise NotImplementedError('Writing predictions is not implemented for this output_mode!')
 
         eval_results.update(eval_result)
 
     if training_args.do_predict:
         logging.info("*** Test ***")
+        test_dataset=dataset.datasets[0]['test']
         # FIXME: this part hasn't been updated for the MTL setup so it doesn't work anymore since
         # predictions is generalized to be a list of predictions and the output needs to be different for each kin.
         # maybe it's ok to only handle classification since it has a very straightforward output format and evaluation,
@@ -699,7 +704,7 @@ def main(json_file=None, json_obj=None):
                 with open(output_test_file, "w") as writer:
                     logger.info("***** Test results *****")
                     for index, item in enumerate(task_predictions):
-                        item = test_dataset.get_labels()[task_ind][item]
+                        item = test_dataset.get_labels()[task_name][item]
                         writer.write("%s\n" % (item))
 
     return eval_results
